# BPP subclass

from balancePowerPerformanceClass import BalancePowerPerformance
import numpy as np
import time


class BPP(BalancePowerPerformance):
    def __init__(self, listMachine, expe, step, resultHeuristicExists):
        """
        The BPP subclass.

        :param listMachine: List containing the machine objects (number, static power, states and name).
        :param expe: Experience object (powerStart, powerEnd et byStep).
        :param step: the step used to vary alpha between 0 and 1
        :param resultHeuristicExists: Boolean. True if the result files of this experiment with these machines
        already exist, False if not.
        """
        super().__init__(listMachine, expe, resultHeuristicExists)
        self.step = step
        self.xRuntime = []

    def getObjectifBpp(self):
        return self.objective

    def getRuntime(self):
        return self.runtime

    def getXRuntime(self):
        return self.xRuntime

    def computeBalancePowerPerformanceHeuristic(self):
        """
        Function that computes the feasible computing power for each given expe power value
        (powerStart, powerEnd, byStep) by varying alpha.
        """
        if not self.resultHeuristicExists:
            start = time.time()
            for power in range(self.expe.powerStart, self.expe.powerEnd, self.expe.byStep):
                start2 = time.time()
                objectiveWithCurrentPower = 0
                for alpha in np.arange(0, 1, self.step):
                    compPower = self.BPPHeuristic(power, alpha)
                    if compPower > objectiveWithCurrentPower:
                        objectiveWithCurrentPower = compPower
                end2 = time.time()
                self.xRuntime.append(end2 - start2)
                self.objective.append(objectiveWithCurrentPower)
            end = time.time()
            self.runtime = end - start
