# Random choice heuristic class

import random
import time
import numpy as np


class RandomChoiceHeuristic:
    def __init__(self, listMachine, expe, resultHeuristicExists):
        """
        The RandomChoiceHeuristic class.

        :param listMachine: List containing the machine objects (number, static power, states and name).
        :param expe: Experience object (powerStart, powerEnd et byStep).
        :param resultHeuristicExists: Boolean. True if the result files of this experiment with these machines
        already exist, False if not.
        """
        self.listMachine = listMachine
        self.expe = expe
        self.resultHeuristicExists = resultHeuristicExists
        self.objectiveAverage = []
        self.objectiveMin = []
        self.objectiveMax = []
        self.runtime = -1
        self.xRuntimeAverage = []

    def getObjectiveAverage(self):
        return self.objectiveAverage

    def getObjectiveMin(self):
        return self.objectiveMin

    def getObjectiveMax(self):
        return self.objectiveMax

    def getRuntime(self):
        return self.runtime

    def getXRuntimeAverage(self):
        return self.xRuntimeAverage

    def randomChoiceHeuristic(self, power):
        """
        The RC heuristic.

        :param power: The available power.
        """
        objectifWithCurrentPower = 0
        numberMachineAvailable = []
        for k in range(len(self.listMachine)):
            numberMachineAvailable.append(self.listMachine[k].number)
        stop = False
        while not stop:
            choiceList = []
            for k in range(len(numberMachineAvailable)):
                if numberMachineAvailable[k] > 0 and power >= self.listMachine[k].static:
                    choiceList.append(self.listMachine[k].name)
            if not choiceList:
                stop = True
            else:
                machineChoose = random.choice(choiceList)
                for k in range(len(self.listMachine)):
                    if machineChoose == self.listMachine[k].name:
                        numberMachineAvailable[k] -= 1
                        power -= self.listMachine[k].static
                        if power >= self.listMachine[k].states[-1][0] * self.listMachine[k].states[-1][1]:
                            objectifWithCurrentPower += self.listMachine[k].states[-1][0]
                            power -= self.listMachine[k].states[-1][0] * self.listMachine[k].states[-1][1]
                        else:
                            maxComp = 0
                            dynPower = 0
                            for i in range(1, len(self.listMachine[k].states)):
                                dynamicPowerMachine = min(self.listMachine[k].states[i][0] *
                                                          self.listMachine[k].states[i][1], power)
                                computingPowerMachine = dynamicPowerMachine / self.listMachine[k].states[i][1]
                                if computingPowerMachine > maxComp:
                                    maxComp = computingPowerMachine
                                    dynPower = dynamicPowerMachine
                            objectifWithCurrentPower += maxComp
                            power -= dynPower
        return objectifWithCurrentPower

    def computeRC(self):
        """
        Function that computes the RC heuristic.
        """
        random.seed(15)
        if not self.resultHeuristicExists:
            start = time.time()
            for power in range(self.expe.powerStart, self.expe.powerEnd, self.expe.byStep):
                runtimeList = []
                objectifList = []
                for i in range(100):
                    start2 = time.time()
                    currentObjectif = self.randomChoiceHeuristic(power)
                    objectifList.append(currentObjectif)
                    end2 = time.time()
                    runtimeList.append(end2 - start2)
                self.xRuntimeAverage.append(np.average(runtimeList))
                self.objectiveAverage.append(np.average(objectifList))
                self.objectiveMin.append((np.min(objectifList)))
                self.objectiveMax.append((np.max(objectifList)))
            end = time.time()
            self.runtime = end - start
